==> kebook2-0.1.1r1.tar.bz2: kebook2-0.1.1r1/kebook2/cebook.hpp <==
/*
 * $Id: cebook.hpp,v 1.3 2001/01/13 19:46:35 hige Exp $
 *
 * Copyright (C) 1999 SUZUKI, Mitsuhiro <hige@higesoft.rim.or.jp>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */
#ifndef CEBOOK_HPP
#define CEBOOK_HPP

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <cstring>
#include <errno.h>
#include <sys/types.h>
#include <sys/stat.h>

// STD
#include <vector>
#include <map>
#include <stdexcept>
#include <string>

#include <eb/eb.h>
#include <eb/text.h>
#include <eb/font.h>
#include <eb/appendix.h>
#include <eb/error.h>

namespace ebook {
    typedef const char*     __string;
    using namespace std;

    const int   RC_ERR = -1;
    class CEBook;
    
    //======== EB Error ========//
    class CEBookError
    {
        friend class CEBook;
    public:
        CEBookError()
            : m_what("No Error"),
              m_eb_error(EB_NO_ERR){}
        CEBookError(const __string& what_arg)
            : m_what(what_arg),
              m_eb_error(::eb_error){}
        CEBookError(const CEBookError& e)
            : m_what(e.what()),
              m_eb_error(e.m_eb_error) {}
        virtual ~CEBookError() {}
        EB_Error_Code getErrorCode() const {return m_eb_error;}
        const string& what() const {return m_what;}
    private:
        void setCurrentError(){
            m_eb_error = ::eb_error;
            m_what = eb_error_message();
        }
        void setNormal() {
            m_eb_error = EB_NO_ERR;
            m_what = eb_error_message2(m_eb_error);
        }
    private:
        string m_what;
        EB_Error_Code m_eb_error;
    };
    //=======================================//
    //======   Class for read EB   ==========//
    //=======================================//
    class CEBook
    {
    public:
        //======== ResultType ========//
        typedef vector< pair< string, string> >     LookupResult;
        typedef LookupResult::value_type            LookupResultElement;

        //======== Search Type ========//
        enum WAY_TO_SEARCH {
            EXACT, FRONT, TAIL
        };
        
        //======== Constructor/Destructor ========//
        CEBook();
        CEBook(const char* dict_path, const char* font_path);
        virtual ~CEBook();
        
        //======== Class (Like ``Interface'' in Java) of client to look up ========//
        class LookupClient
        {
        public:
            virtual void showProgress(int index, int count, int percent) = 0;
            virtual bool isStopped() = 0;
            virtual void enterCriticalSection(void* obj) = 0;
            virtual void leaveCriticalSection(void* obj) = 0;
        };

        
        //======== Look up ========//
        void                lookupWord(LookupClient* client,
                                       LookupResult& result,
                                       const char* word,
                                       WAY_TO_SEARCH way = EXACT);
        void                lookupWord(LookupResult& result,
                                       const char* word,
                                       WAY_TO_SEARCH way = EXACT) {
            lookupWord((LookupClient*)0, result,  word, way);
        }

        void                lookupWord(LookupClient* client,
                                       LookupResult& result,
                                       const string& word,
                                       WAY_TO_SEARCH way = EXACT) {
            lookupWord(client, result, word.c_str(), way);
        }
        
        void                lookupWord(LookupResult& result,
                                       const string& word,
                                       WAY_TO_SEARCH way = EXACT) {
            lookupWord((LookupClient*)0, result, word, way);
        }
        //======== Error ========//
        const CEBookError& getCurrentError() const {return m_error;}
        void setNoError() {m_error.setNormal();}
        
        //======== Get Information ========//
        
        // Information Type //
        struct SubbookInfo {
            string title;
            string directory;
            EB_Subbook_Code code;
        };
        typedef vector <SubbookInfo>                SubbookInfoList;
        typedef SubbookInfoList::const_iterator     SubbookInfoListIterator;
        
        // About Subbook //
        int                                         getSubbook();
        SubbookInfoListIterator                     getBeginOfSubbookInfoList() {
            return m_subbooks->begin();
        }
        SubbookInfoListIterator                     getEndOfSubbookInfoList() {
            return m_subbooks->end();
        }
        void                                        setSubbook(int i) {
            setSubbook(getBeginOfSubbookInfoList()+i);
        }
        void                                        setSubbook(SubbookInfoListIterator it);
        void                                        unsetSubbook() { m_cur_subbook = m_subbooks->end(); }
        const char*                                 getSubbookTitle() const {return (*m_cur_subbook).title.c_str(); }
        const char*                                 getSubbookDirectory();
        const char*                                 getSubbookDirectory(EB_Subbook_Code code);

        // About Other //
        void                getContent(string& content, int page, int offset);

        void                getCopyright(string& copyright);
        void                getDictMenu(string& menu);
        
        const char*         getDictPath() {return eb_path(&m_book);}
        const char*         getFontPath();
        static int          getFontSize();
        static bool         isAvailableDictPath(const char* path);

        inline const char*  getRowStart();
        inline const char*  getRowEnd();
        
        //======== Set Parameter ========//
        void                setFontPath();
        
        void                setDictPath(const char* dict_path);
        void                setFontPath(const char* path);
        static void         setFontSize(int font_size);

    private:
        //======== Read Direct ========//
        void                getHeading(string& heading,  EB_Position* pos);
        void                getContent(string& content, EB_Position* pos);
        
        //======== Set Subbook List ========//
        void                setSubbookInfoList();
        
    private:
        //======== Datas ========//
        EB_Book                     m_book;
        EB_Appendix                 m_appendix;
        EB_Hookset                  m_text_hookset, m_heading_text_hookset;
        SubbookInfoList*            m_subbooks;
        SubbookInfoListIterator     m_cur_subbook;
        int                         m_wrapcol;
        string*                     m_font_path;
        CEBookError                 m_error;
    };
}
#endif // CEBOOK_HPP

==> kebook2-0.2b.tar.bz2: kebook2-0.2b/kebook2/cebook.hpp <==
/*
 * $Id: cebook.hpp,v 1.3.2.1 2001/04/14 22:45:09 hige Exp $
 *
 * Copyright (C) 1999 SUZUKI, Mitsuhiro <hige@higesoft.rim.or.jp>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */
#ifndef CEBOOK_HPP
#define CEBOOK_HPP

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <cstring>
#include <errno.h>
#include <sys/types.h>
#include <sys/stat.h>

// STD
#include <vector>
#include <map>
#include <stdexcept>
#include <string>

#include <eb/eb.h>
#include <eb/text.h>
#include <eb/font.h>
#include <eb/appendix.h>
#include <eb/error.h>

namespace ebook {
    typedef const char*     __string;
    using namespace std;

    const int   RC_ERR = -1;
    const int   EBEX_ERR_NO_KEYWORD_SEARCH = EB_ERR_NO_CANDIDATES+10;
    
    //======== EB Error ========//
    class CEBookError
    {
        friend class CEBook;
    public:
        CEBookError()
            : m_what("No Error"),
              m_eb_error(EB_SUCCESS){}
        /*CEBookError(const __string& what_arg)
            : m_what(what_arg),
            m_eb_error(::eb_error){}*/
        CEBookError(const CEBookError& e)
            : m_what(e.what()),
              m_eb_error(e.m_eb_error) {}
        virtual ~CEBookError() {}
        EB_Error_Code getErrorCode() const {return m_eb_error;}
        const string& what() const {return m_what;}
    private:
        void setError(EB_Error_Code error_code) {
            m_eb_error = error_code;
            if (m_eb_error < EBEX_ERR_NO_KEYWORD_SEARCH)
                m_what = eb_error_message(m_eb_error);
        }
        void setNormal() {
            m_eb_error = EB_SUCCESS;
            m_what = eb_error_message(m_eb_error);
        }
    private:
        string m_what;
        EB_Error_Code m_eb_error;
    };
    class Properties
    {
    public:
        Properties() : m_pos(0), m_page(0) { }
        int getPos() {return m_pos;}
        int getPage() {return m_page;}
    protected:
    private:
        int m_pos, m_page;
    };
    //=======================================//
    //======   Class for read EB   ==========//
    //=======================================//
    class CEBook
    {
    public:
        //======== ResultType ========//
        typedef vector< pair< string, string> >     LookupResult;
        typedef LookupResult::value_type            LookupResultElement;

        //======== Search Type ========//
        enum WAY_TO_SEARCH {
            EXACT, FRONT, TAIL
        };
        
        //======== Initialize/Finalize ========//
        static CEBookError initialize();
        static void finalize() {eb_finalize_library();}

        //======== Constructor/Destructor ========//
        CEBook();
        CEBook(const char* dict_path, const char* font_path);
        virtual ~CEBook();
        
        //======== Class (Like ``Interface'' in Java) of client to look up ========//
        class LookupClient
        {
        public:
            virtual void showProgress(int index, int count, int percent) = 0;
            virtual bool isStopped() = 0;
            virtual void enterCriticalSection(void* obj) = 0;
            virtual void leaveCriticalSection(void* obj) = 0;
        };

        //======== Look up ========//
        void                lookupWord(LookupClient* client,
                                       LookupResult& result,
                                       const char* word,
                                       WAY_TO_SEARCH way = EXACT);
        void                lookupWord(LookupResult& result,
                                       const char* word,
                                       WAY_TO_SEARCH way = EXACT) {
            lookupWord((LookupClient*)0, result,  word, way);
        }

        void                lookupWord(LookupClient* client,
                                       LookupResult& result,
                                       const string& word,
                                       WAY_TO_SEARCH way = EXACT) {
            lookupWord(client, result, word.c_str(), way);
        }
        
        void                lookupWord(LookupResult& result,
                                       const string& word,
                                       WAY_TO_SEARCH way = EXACT) {
            lookupWord((LookupClient*)0, result, word, way);
        }
        //======== Error ========//
        const CEBookError& getCurrentError() const {return m_error;}
        void setNoError() {m_error.setNormal();}
       
        //======== Get Information ========//
        // Information Type //
        struct SubbookInfo {
            string title;
            string directory;
            EB_Subbook_Code code;
        };
        typedef vector <SubbookInfo>                SubbookInfoList;
        typedef SubbookInfoList::const_iterator     SubbookInfoListIterator;
        
        // About Subbook //
        int                                         getSubbook();
        SubbookInfoListIterator                     getBeginOfSubbookInfoList() {
            return m_subbooks->begin();
        }
        SubbookInfoListIterator                     getEndOfSubbookInfoList() {
            return m_subbooks->end();
        }
        void                                        setSubbook(int i) {
            setSubbook(getBeginOfSubbookInfoList()+i);
        }
        void                                        unsetSubbook();
        void                                        setSubbook(SubbookInfoListIterator it);
        const char*                                 getSubbookTitle() const {
            return (*m_cur_subbook).title.c_str();
        }
        const char*                                 getSubbookDirectory();
        const char*                                 getSubbookDirectory(EB_Subbook_Code code);

        // About Other //
        void                getContent(string& content, int page, int offset);

        void                getCopyright(string& copyright);
        void                getDictMenu(string& menu);
        
        const char*         getDictPath() {return m_dict_path->c_str();}
        const char*         getFontPath();
        static int          getFontSize();
        static bool         isAvailableDictPath(const char* path);

        inline const char*  getRowStart();
        inline const char*  getRowEnd();
        
        //======== Set Parameter ========//
        void                setFontPath();
        
        void                setDictPath(const char* dict_path);
        void                setFontPath(const char* path);
        static void         setFontSize(int font_size);

    private:
        //======== Read Direct ========//
        void                getHeading(string& heading,  EB_Position* pos);
        void                getContent(string& content, EB_Position* pos);
        
        //======== Set Subbook List ========//
        void                setSubbookInfoList();
        
    private:
        //======== Datas ========//
        EB_Book                     m_book;
        EB_Appendix                 m_appendix;
        EB_Hookset                  m_text_hookset, m_heading_text_hookset;
        vector<EB_Position>         m_pos_array;
        SubbookInfoList*            m_subbooks;
        SubbookInfoListIterator     m_cur_subbook;
        int                         m_wrapcol;
        string*                     m_dict_path;
        string*                     m_font_path;
        CEBookError                 m_error;
    };
}
#endif // CEBOOK_HPP
